
#include "lin.h"
#include "at32f421_board.h"

static lin_work_mode_t lin_work_mode = LIN_IDLE;

static lin_state_t lin_state = LIN_SLAVE_IDLE;

//static lin_master_state_t lin_master_state = LIN_MASTER_IDLE;
static u8 lin_master_rxdata_count;
static lin_frame_slot_t lin_master_slot  = {0};
static bool             lin_master_rx_ready = false;
//static bool             lin_master_tx_ready = false;

//static lin_slave_state_t lin_slave_state = LIN_SLAVE_IDLE;
static u8 lin_slave_rxdata_count;
static lin_frame_slot_t lin_slave_slot  = {0};
static bool             lin_slave_rx_ready = false;
static bool             lin_slave_tx_ready = false;

void lin_hw_init(void)
{
  gpio_init_type gpio_init_struct;

  /* enable the uart1 and gpio clock */
  crm_periph_clock_enable(CRM_USART2_PERIPH_CLOCK, TRUE);
  crm_periph_clock_enable(CRM_GPIOA_PERIPH_CLOCK, TRUE);

  gpio_default_para_init(&gpio_init_struct);

  /* configure the uart1 tx pin */
  gpio_init_struct.gpio_drive_strength = GPIO_DRIVE_STRENGTH_STRONGER;
  gpio_init_struct.gpio_out_type  = GPIO_OUTPUT_PUSH_PULL;
  gpio_init_struct.gpio_mode = GPIO_MODE_MUX;
  gpio_init_struct.gpio_pins = GPIO_PINS_2 | GPIO_PINS_3;
  gpio_init_struct.gpio_pull = GPIO_PULL_NONE;
  gpio_init(GPIOA, &gpio_init_struct);

  gpio_pin_mux_config(GPIOA, GPIO_PINS_SOURCE2, GPIO_MUX_1);
  gpio_pin_mux_config(GPIOA, GPIO_PINS_SOURCE3, GPIO_MUX_1);
  
//  USART_InitType USART_InitStructure;
//  
//  USART_StructInit(&USART_InitStructure);
//  USART_InitStructure.USART_BaudRate = LIN_USART_BAUDRATE;
//  USART_InitStructure.USART_WordLength = USART_WordLength_8b;
//  USART_InitStructure.USART_StopBits = USART_StopBits_1;
//  USART_InitStructure.USART_Parity = USART_Parity_No;
//  USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
//  USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;
//  USART_Init(LIN_USART, &USART_InitStructure);
  usart_init(LIN_USART, LIN_USART_BAUDRATE, USART_DATA_8BITS, USART_STOP_1_BIT);
  
//  USART_LINBreakDetectLengthConfig(LIN_USART, USART_LINBreakDetectLength_11b);
//  USART_LINCmd(LIN_USART, ENABLE);
  usart_break_bit_num_set(LIN_USART, USART_BREAK_11BITS);
  usart_lin_mode_enable(LIN_USART, TRUE);
//  USART_Cmd(LIN_USART, ENABLE);
  usart_transmitter_enable(LIN_USART, TRUE);
  usart_receiver_enable(LIN_USART, TRUE);
  usart_enable(LIN_USART, TRUE);
}

bool lin_hw_check_for_break(void)
{
	bool result = usart_flag_get(LIN_USART, USART_BFF_FLAG);
	if (result == true)
	{
		usart_flag_clear(LIN_USART, USART_BFF_FLAG|USART_RDBF_FLAG);
	}
  
	return result;
}

bool lin_hw_check_for_data(u8 *byte)
{
	bool result = usart_flag_get(LIN_USART, USART_RDBF_FLAG);
	if (result == true)
	{
	  *byte = usart_data_receive(LIN_USART);
	}
  
	return result;
}

bool lin_hw_tx_byte(u8 byte)
{
  while(usart_flag_get(LIN_USART, USART_TDBE_FLAG) == RESET);
  usart_data_transmit(LIN_USART, byte);
    
	return true;
}

bool lin_hw_tx_break(void)
{
	usart_break_send(LIN_USART);

	return true;
}

bool lin_hw_tx_data(u8* data, u8 len)
{
	while(len--){
    while(usart_flag_get(LIN_USART, USART_TDBE_FLAG) == RESET);
    usart_data_transmit(LIN_USART, *data++);
  }
  
  return true;
}

u8 lin_data_layer_parity(u8 pid) 
{
	u8 result = 0;
	result = (u8)(LIN_GET_PID_BIT(pid,0) ^ LIN_GET_PID_BIT(pid,1) ^
			 LIN_GET_PID_BIT(pid,2) ^ LIN_GET_PID_BIT(pid,4)) << LIN_P0_FLAG;

  result |= (u8)(~(LIN_GET_PID_BIT(pid,1) ^ LIN_GET_PID_BIT(pid,3) ^
    			  LIN_GET_PID_BIT(pid,4) ^ LIN_GET_PID_BIT(pid,5)) << LIN_P1_FLAG);

  result |= (u8)(LIN_ID_MASK & pid);
  return result;
}

u8 lin_data_layer_checksum(u8 pid, u8 lenght, const u8* data_ptr)
{
	u8 i = 0;
	u16 result;

	/* diagnostic request have classic checksum */
	if ((pid == LIN_DIAG_REQUEST) || (pid == LIN_DIAG_RESPONSE))
	{
		result = 0;
	} 
  else 
  {
		result = lin_data_layer_parity(pid);
	}

	for (i = 0; i < lenght; i++)
	{
		result += data_ptr[i];
		if (result > 0xFF)
		{
			result -= 0xFF;
		} 
    else
		{
			/* do nothing */
		}
	}
	return (u8)(~result);
}

void lin_master_rx_reset(void)
{
  lin_state = LIN_MASTER_DATA_RX;
	lin_master_rxdata_count = 0;
}

void lin_slave_rx_reset(void)
{
	lin_state = LIN_SLAVE_IDLE;
	lin_slave_rxdata_count = 0;
}

void lin_tx_head(u8 pid)
{
  lin_hw_tx_break();
  lin_hw_tx_byte(LIN_SYNCH_BYTE);
  lin_hw_tx_byte(lin_data_layer_parity(pid));
}

void lin_set_rx_enabled(bool v)
{
	if (v == true)
	{
    usart_flag_clear(LIN_USART, USART_RDBF_FLAG|USART_BFF_FLAG);
		usart_interrupt_enable(LIN_USART, USART_BF_INT, TRUE);
    usart_interrupt_enable(LIN_USART, USART_RDBF_INT, TRUE);
    NVIC_EnableIRQ(LIN_USART_IRQ);
	} 
  else
	{
    usart_interrupt_enable(LIN_USART, USART_BF_INT, FALSE);
    usart_interrupt_enable(LIN_USART, USART_RDBF_INT, FALSE);
	}
}

bool lin_master_handler(lin_frame_slot_t *f)
{
  u8 cnt = f->data_length;
  u8*buf = f->data_ptr; 
  
  lin_work_mode = LIN_MASTER;
  memcpy(&lin_master_slot, f, sizeof(lin_frame_slot_t));
  lin_set_rx_enabled(true);
  lin_master_rx_reset();
  
	lin_tx_head(f->pid);
  
  if(f->frame_type == LIN_FRAME_TYPE_TRANSMIT){
    delay_ms(1);  //Response space
    while(cnt--){
      lin_hw_tx_byte(*buf++);
    }
    lin_hw_tx_byte(lin_data_layer_checksum(f->pid, f->data_length, f->data_ptr));
    lin_set_rx_enabled(false);
    memset(&lin_master_slot, 0, sizeof(lin_frame_slot_t));
    lin_work_mode = LIN_IDLE;
    return true;
  }
  if(f->frame_type == LIN_FRAME_TYPE_RECEIVE){
    while(false == lin_master_rx_ready);  //ȴslaveӦ
    lin_master_rx_ready = false;
    memcpy(f, &lin_master_slot, sizeof(lin_frame_slot_t));
    lin_set_rx_enabled(false);
    memset(&lin_master_slot, 0, sizeof(lin_frame_slot_t));
    lin_work_mode = LIN_IDLE;
    return true;
  }
  
  lin_set_rx_enabled(false);
  memset(&lin_master_slot, 0, sizeof(lin_frame_slot_t));
  lin_work_mode = LIN_IDLE;
  return false;
}

bool lin_slave_handler(lin_frame_slot_t *f)
{
  u8 cnt = f->data_length;
  u8*buf = f->data_ptr;
  
  lin_work_mode = LIN_SLAVE;
  memcpy(&lin_slave_slot, f, sizeof(lin_frame_slot_t));
  lin_set_rx_enabled(true);
  lin_slave_rx_reset();
  
  if(LIN_FRAME_TYPE_RECEIVE == lin_slave_slot.frame_type)
  {
    while(false == lin_slave_rx_ready);  //յ
    lin_slave_rx_ready = false;
    memcpy(f, &lin_slave_slot, sizeof(lin_frame_slot_t));
    lin_set_rx_enabled(false);
    memset(&lin_slave_slot, 0, sizeof(lin_frame_slot_t));
    lin_work_mode = LIN_IDLE;
    return true;
  }
  else if(LIN_FRAME_TYPE_TRANSMIT == lin_slave_slot.frame_type)
  {
    while(false == lin_slave_tx_ready);  //յ֡ͷ
    lin_slave_tx_ready = false;
    while(cnt--){
      lin_hw_tx_byte(*buf++);
    }
    lin_hw_tx_byte(lin_data_layer_checksum(f->pid, f->data_length, f->data_ptr));
    lin_set_rx_enabled(false);
    memset(&lin_slave_slot, 0, sizeof(lin_frame_slot_t));
    lin_work_mode = LIN_IDLE;
    return true;
  }
  
  lin_set_rx_enabled(false);
  memset(&lin_slave_slot, 0, sizeof(lin_frame_slot_t));
  lin_work_mode = LIN_IDLE;
  return false;
}

void LIN_USART_IRQHandle(void)
{
  u8 rx_byte;
  
  if(LIN_SLAVE == lin_work_mode){
    if(lin_hw_check_for_break() == true)
    {
      lin_state = LIN_SLAVE_SYNC_RX;
    } 
    if(lin_hw_check_for_data(&rx_byte) == true)
    {
      switch (lin_state){
        case (LIN_SLAVE_SYNC_RX):
        {
          /* synch byte reception do nothing */
          if (rx_byte != LIN_SYNCH_BYTE)
          {
            lin_slave_rx_reset();
          } 
          else
          {
            lin_state = LIN_SLAVE_PID_RX;
          }
          break;
        }
        
        case (LIN_SLAVE_PID_RX):
        {
          if (rx_byte == 0) 
            break;
          if (lin_data_layer_parity(rx_byte) == rx_byte)
          {
            if(LIN_FRAME_TYPE_RECEIVE == lin_slave_slot.frame_type)
            {
              lin_slave_slot.pid = (u8)(rx_byte & LIN_ID_MASK);
              lin_state = LIN_SLAVE_DATA_RX;
            }
            else if(LIN_FRAME_TYPE_TRANSMIT == lin_slave_slot.frame_type)
            {
              if(lin_slave_slot.pid == (u8)(rx_byte & LIN_ID_MASK)){  //ַƥӦ
                lin_state = LIN_SLAVE_DATA_TX;
                lin_slave_tx_ready = true;
                lin_slave_rx_reset();
              }
              else{
                lin_slave_rx_reset();
              }
            }
          } 
          else
          {
            lin_slave_rx_reset();
          }
          break;
        }

        case (LIN_SLAVE_DATA_RX):
        {
          if (lin_slave_rxdata_count < lin_slave_slot.data_length)
          {
            lin_slave_slot.data_ptr[lin_slave_rxdata_count] = rx_byte;
            lin_slave_rxdata_count ++;
          } 
          else /* 9th byte is SUM */
          {
            lin_slave_slot.data_length = lin_slave_rxdata_count;
            /* checksum calculation */
            if (rx_byte == lin_data_layer_checksum(lin_slave_slot.pid & LIN_ID_MASK,
                lin_slave_slot.data_length, lin_slave_slot.data_ptr)) 
            {
              /* valid checksum */
              lin_slave_rx_ready = true;
            }
            lin_slave_rx_reset();
          }
          break;
        }
        
        default:
          lin_slave_rx_reset();
          break;
      }
    }
  }
	else if(LIN_MASTER == lin_work_mode){
    if(lin_hw_check_for_data(&rx_byte) == true)
    {
      switch (lin_state){
        case (LIN_MASTER_DATA_RX):
        {
          if (lin_master_rxdata_count < lin_master_slot.data_length)
          {
            lin_master_slot.data_ptr[lin_master_rxdata_count] = rx_byte;
            lin_master_rxdata_count ++;
          } 
          else /* 9th byte is SUM */
          {
            lin_master_slot.data_length = lin_master_rxdata_count;
            /* checksum calculation */
            if (rx_byte == lin_data_layer_checksum(lin_master_slot.pid & LIN_ID_MASK,
                lin_master_slot.data_length, lin_master_slot.data_ptr)) 
            {
              /* valid checksum */
              lin_master_rx_ready = true;
            }
            lin_master_rx_reset();
          }
          break;
        }

        default:
          lin_master_rx_reset();
          break;
      }
    }
  }
}

