/*
 * lin.h
 *
 */

#ifndef LIN_H_
#define LIN_H_

#include <stdbool.h>
#include <string.h>
#include "at32f421.h"

#define LIN_USART              USART2
#define LIN_USART_BAUDRATE     19200
#define LIN_USART_IRQ          USART2_IRQn
#define LIN_USART_IRQHandle    USART2_IRQHandler

#define LIN_MAX_FRAME_LENGTH 0x8u
#define LIN_SYNCH_BYTE 0x55u
#define LIN_GET_PID_BIT(x,y) (((x) >> (y)) & 0x01u)
#define LIN_ID_MASK 0x3Fu
#define LIN_P0_FLAG 6
#define LIN_P1_FLAG 7

#define LIN_DIAG_REQUEST 0x3C
#define LIN_DIAG_RESPONSE 0x3D

typedef enum{
  LIN_IDLE,
	LIN_MASTER,
	LIN_SLAVE,
} lin_work_mode_t;

//typedef enum {
//	LIN_SLAVE_IDLE,
//	LIN_SLAVE_SYNC_RX,
//	LIN_SLAVE_PID_RX,
//	LIN_SLAVE_DATA_RX,
//	LIN_SLAVE_DATA_TX
//} lin_slave_state_t;

//typedef enum {
//	LIN_MASTER_IDLE,
//	LIN_MASTER_DATA_RX,
//	LIN_MASTER_DATA_TX,
//} lin_master_state_t;

typedef enum {
	LIN_SLAVE_IDLE,
	LIN_SLAVE_SYNC_RX,
	LIN_SLAVE_PID_RX,
	LIN_SLAVE_DATA_RX,
	LIN_SLAVE_DATA_TX, 
	LIN_MASTER_IDLE,
	LIN_MASTER_DATA_RX,
	LIN_MASTER_DATA_TX,
} lin_state_t;

typedef enum{
	LIN_FRAME_TYPE_TRANSMIT,
	LIN_FRAME_TYPE_RECEIVE
} lin_frame_type_t;

typedef struct{
	u8 pid;
	lin_frame_type_t frame_type;
	u8 data_length;
	u8 *data_ptr;
}lin_frame_slot_t;

void lin_hw_init(void);
void lin_set_rx_enabled(bool v);

bool lin_master_handler(lin_frame_slot_t *f);
bool lin_slave_handler(lin_frame_slot_t *f);

#endif /* LIN_SLCAN_H_ */
